@@ -1,37 +1,33 @@
 /*
-    Copyright (C) 2010-2011 wpitchoune@gmail.com
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-    02110-1301 USA
-*/
-
+ * Copyright (C) 2010-2011 jeanfi@gmail.com
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301 USA
+ */
 #include <locale.h>
-#include <libintl.h>
-#define _(str) gettext(str)
 
 #include <getopt.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include <gtk/gtk.h>
 
-#include <sensors/sensors.h>
-#include <sensors/error.h>
-
 #include "config.h"
 
 #include "cfg.h"
@@ -42,39 +38,52 @@
 #include "ui_color.h"
 #include "lmsensor.h"
 #include "ui_pref.h"
+#include "ui_graph.h"
+#include "ui_status.h"
+
+#ifdef HAVE_UNITY
+#include "ui_unity.h"
+#endif
 
 #ifdef HAVE_NVIDIA
 #include "nvidia.h"
 #endif
 
+#ifdef HAVE_LIBATIADL
+#include "amd.h"
+#endif
+
 #ifdef HAVE_REMOTE_SUPPORT
 #include "rsensor.h"
 #endif
 
-#if defined(HAVE_APPINDICATOR) || defined(HAVE_APPINDICATOR_029)
 #include "ui_appindicator.h"
-#endif
 
 #ifdef HAVE_LIBNOTIFY
 #include "ui_notify.h"
 #endif
 
+#ifdef HAVE_GTOP
+#include "cpu.h"
+#endif
+
 #include "compat.h"
 
 static const char *program_name;
 
-void print_version()
+static void print_version()
 {
 	printf("psensor %s\n", VERSION);
-	printf(_("Copyright (C) %s wpitchoune@gmail.com\n\
-License GPLv2: GNU GPL version 2 or later \
-<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>\n\
-This is free software: you are free to change and redistribute it.\n\
-There is NO WARRANTY, to the extent permitted by law.\n"),
+	printf(_("Copyright (C) %s jeanfi@gmail.com\n"
+		 "License GPLv2: GNU GPL version 2 or later "
+		 "<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>\n"
+		 "This is free software: you are free to change and "
+		 " redistribute it.\n"
+		 "There is NO WARRANTY, to the extent permitted by law.\n"),
 	       "2010-2011");
 }
 
-void print_help()
+static void print_help()
 {
 	printf(_("Usage: %s [OPTION]...\n"), program_name);
 
@@ -82,16 +91,19 @@ void print_help()
 	       "including temperatures and fan speeds."));
 
 	puts("");
-	puts("Options:");
-	puts(_("\
-  -h, --help          display this help and exit\n\
-  -v, --version       display version information and exit"));
+	puts(_("Options:"));
+	puts(_("  -h, --help          display this help and exit\n"
+	       "  -v, --version       display version information and exit"));
 
 	puts("");
 
-	puts(_("\
-  -u, --url=URL       \
-the URL of the psensor-server, example: http://hostname:3131"));
+	puts(_("  -u, --url=URL       "
+	       "the URL of the psensor-server, example: http://hostname:3131"));
+
+	puts("");
+
+	puts(_("  -d, --debug=LEVEL   "
+	       "set the debug level, integer between 0 and 3"));
 
 	puts("");
 
@@ -100,25 +112,6 @@ the URL of the psensor-server, example: http://hostname:3131"));
 	printf(_("%s home page: <%s>\n"), PACKAGE_NAME, PACKAGE_URL);
 }
 
-static void
-cb_preferences(gpointer data, guint callback_action, GtkWidget *item)
-{
-	ui_pref_dialog_run((struct ui_psensor *)data);
-}
-
-static GtkItemFactoryEntry menu_items[] = {
-	{"/Preferences",
-	 NULL, cb_preferences, 0, "<Item>"},
-
-	{"/sep1",
-	 NULL, NULL, 0, "<Separator>"},
-
-	{"/Quit",
-	 "", ui_psensor_exit, 0, "<StockItem>", GTK_STOCK_QUIT},
-};
-
-static gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
-
 /*
   Updates the size of the sensor values if different than the
   configuration.
@@ -140,13 +133,25 @@ update_psensor_values_size(struct psensor **sensors, struct config *cfg)
 	}
 }
 
+static void log_measures(struct psensor **sensors)
+{
+	if (log_level == LOG_DEBUG)
+		while (*sensors) {
+			log_debug("Measure: %s %.2f",
+				   (*sensors)->name,
+				   psensor_get_current_value(*sensors));
+
+			sensors++;
+		}
+}
+
 void update_psensor_measures(struct ui_psensor *ui)
 {
 	struct psensor **sensors = ui->sensors;
 	struct config *cfg = ui->config;
 
 	while (1) {
-		gdk_threads_enter();
+		g_mutex_lock(ui->sensors_mutex);
 
 		if (!sensors)
 			return;
@@ -160,13 +165,43 @@ void update_psensor_measures(struct ui_psensor *ui)
 #ifdef HAVE_NVIDIA
 		nvidia_psensor_list_update(sensors);
 #endif
+#ifdef HAVE_LIBATIADL
+		amd_psensor_list_update(sensors);
+#endif
 
-		gdk_threads_leave();
+		log_measures(sensors);
+
+		g_mutex_unlock(ui->sensors_mutex);
 
 		sleep(cfg->sensor_update_interval);
 	}
 }
 
+static void indicators_update(struct ui_psensor *ui)
+{
+	struct psensor **sensor_cur = ui->sensors;
+	unsigned int attention = 0;
+
+	while (*sensor_cur) {
+		struct psensor *s = *sensor_cur;
+
+		if (s->alarm_enabled && s->alarm_raised) {
+			attention = 1;
+			break;
+		}
+
+		sensor_cur++;
+	}
+
+#if defined(HAVE_APPINDICATOR) || defined(HAVE_APPINDICATOR_029)
+	if (is_appindicator_supported())
+		ui_appindicator_update(ui, attention);
+#endif
+
+	if (is_status_supported())
+		ui_status_update(ui, attention);
+}
+
 gboolean ui_refresh_thread(gpointer data)
 {
 	struct config *cfg;
@@ -176,14 +211,18 @@ gboolean ui_refresh_thread(gpointer data)
 	ret = TRUE;
 	cfg = ui->config;
 
-	gdk_threads_enter();
+	g_mutex_lock(ui->sensors_mutex);
 
 	graph_update(ui->sensors, ui->w_graph, ui->config);
 
-	ui_sensorlist_update(ui->ui_sensorlist);
+	ui_sensorlist_update(ui);
 
-#if defined(HAVE_APPINDICATOR) || defined(HAVE_APPINDICATOR_029)
-	ui_appindicator_update(ui);
+	if (is_appindicator_supported() || is_status_supported())
+		indicators_update(ui);
+
+#ifdef HAVE_UNITY
+	ui_unity_launcher_entry_update(ui->sensors,
+				       !cfg->unity_launcher_count_disabled);
 #endif
 
 	if (ui->graph_update_interval != cfg->graph_update_interval) {
@@ -191,7 +230,7 @@ gboolean ui_refresh_thread(gpointer data)
 		ret = FALSE;
 	}
 
-	gdk_threads_leave();
+	g_mutex_unlock(ui->sensors_mutex);
 
 	if (ret == FALSE)
 		g_timeout_add(1000 * ui->graph_update_interval,
@@ -200,17 +239,6 @@ gboolean ui_refresh_thread(gpointer data)
 	return ret;
 }
 
-gboolean
-on_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
-{
-	struct ui_psensor *ui_psensor = (struct ui_psensor *)data;
-
-	graph_update(ui_psensor->sensors,
-		     ui_psensor->w_graph, ui_psensor->config);
-
-	return FALSE;
-}
-
 void cb_alarm_raised(struct psensor *sensor, void *data)
 {
 #ifdef HAVE_LIBNOTIFY
@@ -219,7 +247,7 @@ void cb_alarm_raised(struct psensor *sensor, void *data)
 #endif
 }
 
-void associate_colors(struct psensor **sensors)
+static void associate_colors(struct psensor **sensors)
 {
 	/* number of uniq colors */
 #define COLORS_COUNT 8
@@ -254,7 +282,8 @@ void associate_colors(struct psensor **sensors)
 	}
 }
 
-void associate_cb_alarm_raised(struct psensor **sensors, struct ui_psensor *ui)
+static void
+associate_cb_alarm_raised(struct psensor **sensors, struct ui_psensor *ui)
 {
 	struct psensor **sensor_cur = sensors;
 	while (*sensor_cur) {
@@ -277,7 +306,7 @@ void associate_cb_alarm_raised(struct psensor **sensors, struct ui_psensor *ui)
 	}
 }
 
-void associate_preferences(struct psensor **sensors)
+static void associate_preferences(struct psensor **sensors)
 {
 	struct psensor **sensor_cur = sensors;
 	while (*sensor_cur) {
@@ -295,114 +324,65 @@ void associate_preferences(struct psensor **sensors)
 	}
 }
 
-GtkWidget *ui_get_popupmenu(gpointer data)
+static void log_init()
 {
-	GtkItemFactory *item_factory;
-	GtkWidget *menu;
+	char *home, *path, *dir;
 
-	item_factory = gtk_item_factory_new(GTK_TYPE_MENU, "<main>", NULL);
-	gtk_item_factory_create_items(item_factory,
-				      nmenu_items, menu_items, data);
-	menu = gtk_item_factory_get_widget(item_factory, "<main>");
+	home = getenv("HOME");
 
-	return menu;
-}
+	if (!home)
+		return ;
 
-int on_graph_clicked(GtkWidget *widget, GdkEventButton *event, gpointer data)
-{
-	GtkWidget *menu;
+	dir = malloc(strlen(home)+1+strlen(".psensor")+1);
+	sprintf(dir, "%s/%s", home, ".psensor");
+	mkdir(dir, 0777);
 
-	if (event->type != GDK_BUTTON_PRESS)
-		return FALSE;
+	path = malloc(strlen(dir)+1+strlen("log")+1);
+	sprintf(path, "%s/%s", dir, "log");
 
-	menu = ui_get_popupmenu(data);
+	log_open(path);
 
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-		       event->button, event->time);
-
-	return TRUE;
-}
-
-
-GtkWidget *create_graph_widget(struct ui_psensor * ui)
-{
-	GtkWidget *w_graph;
-
-	w_graph = gtk_drawing_area_new();
-
-	g_signal_connect(G_OBJECT(w_graph),
-			 "expose-event", G_CALLBACK(on_expose_event), ui);
-
-	gtk_widget_add_events(w_graph, GDK_BUTTON_PRESS_MASK);
-	gtk_signal_connect(GTK_OBJECT(w_graph),
-			   "button_press_event",
-			   (GCallback) on_graph_clicked, ui);
-
-	return w_graph;
-}
-
-void ui_main_box_create(struct ui_psensor *ui)
-{
-	struct config *cfg;
-	GtkWidget *w_sensorlist;
-
-	cfg = ui->config;
-
-	if (ui->main_box) {
-		ui_sensorlist_create_widget(ui->ui_sensorlist);
-
-		gtk_container_remove(GTK_CONTAINER(ui->main_window),
-				     ui->main_box);
-
-		ui->w_graph = create_graph_widget(ui);
-		ui->w_sensorlist = ui->ui_sensorlist->widget;
-	}
-
-	if (cfg->sensorlist_position == SENSORLIST_POSITION_RIGHT
-	    || cfg->sensorlist_position == SENSORLIST_POSITION_LEFT)
-		ui->main_box = gtk_hpaned_new();
-	else
-		ui->main_box = gtk_vpaned_new();
-
-	w_sensorlist = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w_sensorlist),
-				       GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_container_add(GTK_CONTAINER(w_sensorlist),
-			  ui->ui_sensorlist->widget);
-
-	gtk_container_add(GTK_CONTAINER(ui->main_window), ui->main_box);
-
-	if (cfg->sensorlist_position == SENSORLIST_POSITION_RIGHT
-	    || cfg->sensorlist_position == SENSORLIST_POSITION_BOTTOM) {
-		gtk_paned_pack1(GTK_PANED(ui->main_box),
-				GTK_WIDGET(ui->w_graph), TRUE, TRUE);
-		gtk_paned_pack2(GTK_PANED(ui->main_box),
-				w_sensorlist, FALSE, TRUE);
-	} else {
-		gtk_paned_pack1(GTK_PANED(ui->main_box),
-				w_sensorlist, FALSE, TRUE);
-		gtk_paned_pack2(GTK_PANED(ui->main_box),
-				GTK_WIDGET(ui->w_graph), TRUE, TRUE);
-	}
-
-	gtk_widget_show_all(ui->main_box);
+	free(dir);
+	free(path);
 }
 
 static struct option long_options[] = {
 	{"version", no_argument, 0, 'v'},
 	{"help", no_argument, 0, 'h'},
 	{"url", required_argument, 0, 'u'},
+	{"debug", required_argument, 0, 'd'},
 	{0, 0, 0, 0}
 };
 
+static gboolean initial_window_show(gpointer data)
+{
+	struct ui_psensor *ui;
+
+	log_debug("initial_window_show()");
+
+	ui = (struct ui_psensor *)data;
+
+	log_debug("is_status_supported: %d", is_status_supported());
+	log_debug("is_appindicator_supported: %d",
+		   is_appindicator_supported());
+	log_debug("hide_on_startup: %d", ui->config->hide_on_startup);
+
+	if (!ui->config->hide_on_startup
+	    || (!is_appindicator_supported() && !is_status_supported()))
+		ui_window_show(ui);
+
+	ui_window_update(ui);
+
+	return FALSE;
+}
+
 int main(int argc, char **argv)
 {
 	struct ui_psensor ui;
 	GError *error;
 	GThread *thread;
-	int err, optc;
+	int optc, cmdok;
 	char *url = NULL;
-	int cmdok = 1;
 
 	program_name = argv[0];
 
@@ -413,7 +393,8 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 #endif
 
-	while ((optc = getopt_long(argc, argv, "vhu:", long_options,
+	cmdok = 1;
+	while ((optc = getopt_long(argc, argv, "vhd:u:", long_options,
 				   NULL)) != -1) {
 		switch (optc) {
 		case 'u':
@@ -426,6 +407,10 @@ int main(int argc, char **argv)
 		case 'v':
 			print_version();
 			exit(EXIT_SUCCESS);
+		case 'd':
+			log_level = atoi(optarg);
+			log_printf(LOG_INFO, _("Enables debug mode."));
+			break;
 		default:
 			cmdok = 0;
 			break;
@@ -438,26 +423,21 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	log_init();
+
 	g_thread_init(NULL);
 	gdk_threads_init();
-	gdk_threads_enter();
+	/* gdk_threads_enter(); */
 
-	gtk_init(&argc, &argv);
+	gtk_init(NULL, NULL);
 
-#ifdef HAVE_LIBNOTIFY
-	ui.notification_last_time = NULL;
-#endif
+	ui.sensors_mutex = g_mutex_new();
 
 	config_init();
 
 	ui.config = config_load();
 
-	err = lmsensor_init();
-	if (!err) {
-		fprintf(stderr, _("ERROR: lmsensor init failure: %s\n"),
-			sensors_strerror(err));
-		exit(EXIT_FAILURE);
-	}
+	psensor_init();
 
 	if (url) {
 #ifdef HAVE_REMOTE_SUPPORT
@@ -469,16 +449,15 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 #endif
 	} else {
-#ifdef HAVE_NVIDIA
-		struct psensor **tmp;
-
-		tmp = get_all_sensors(600);
-		ui.sensors = nvidia_psensor_list_add(tmp, 600);
-
-		if (tmp != ui.sensors)
-			free(tmp);
-#else
 		ui.sensors = get_all_sensors(600);
+#ifdef HAVE_NVIDIA
+		ui.sensors = nvidia_psensor_list_add(ui.sensors, 600);
+#endif
+#ifdef HAVE_LIBATIADL
+		ui.sensors = amd_psensor_list_add(ui.sensors, 600);
+#endif
+#ifdef HAVE_GTOP
+		ui.sensors = cpu_psensor_list_add(ui.sensors, 600);
 #endif
 	}
 
@@ -486,19 +465,19 @@ int main(int argc, char **argv)
 	associate_colors(ui.sensors);
 	associate_cb_alarm_raised(ui.sensors, &ui);
 
+#if !defined(HAVE_APPINDICATOR) && !defined(HAVE_APPINDICATOR_029)
+	ui_status_init(&ui);
+#endif
+
 	/* main window */
-	ui.main_window = ui_window_create(&ui);
-	ui.main_box = NULL;
+	ui_window_create(&ui);
+	ui.sensor_box = NULL;
 
 	/* drawing box */
-	ui.w_graph = create_graph_widget(&ui);
+	ui.w_graph = ui_graph_create(&ui);
 
 	/* sensor list */
-	ui.ui_sensorlist = ui_sensorlist_create(ui.sensors);
-
-	ui_main_box_create(&ui);
-
-	gtk_widget_show_all(ui.main_window);
+	ui_sensorlist_create(&ui);
 
 	thread = g_thread_create((GThreadFunc) update_psensor_measures,
 				 &ui, TRUE, &error);
@@ -514,14 +493,47 @@ int main(int argc, char **argv)
 	ui_appindicator_init(&ui);
 #endif
 
+	/*
+	 * show the window as soon as all gtk events have been processed
+	 * in order to ensure that the status icon is attempted to be
+	 * drawn before. If not, there is no way to detect that it is
+	 * visible.
+	*/
+	g_idle_add((GSourceFunc)initial_window_show, &ui);
+
+	gdk_notify_startup_complete();
+
 	/* main loop */
 	gtk_main();
 
-	sensors_cleanup();
+	g_mutex_lock(ui.sensors_mutex);
+
+	psensor_cleanup();
+
+#ifdef HAVE_NVIDIA
+	nvidia_cleanup();
+#endif
+#ifdef HAVE_LIBATIADL
+	amd_cleanup();
+#endif
+#ifdef HAVE_REMOTE_SUPPORT
+	rsensor_cleanup();
+#endif
 
 	psensor_list_free(ui.sensors);
+	ui.sensors = NULL;
 
-	gdk_threads_leave();
+#if defined(HAVE_APPINDICATOR) || defined(HAVE_APPINDICATOR_029)
+	ui_appindicator_cleanup();
+#endif
+
+	ui_status_cleanup();
+
+	g_mutex_unlock(ui.sensors_mutex);
+
+	config_cleanup();
+
+	log_close();
 
 	return 0;
 }
